static const char rcsid[]="$Id: gdbmobj.C,v 1.2 1999/02/18 01:06:48 mrsam Exp $";

#include	"gdbmobj.h"


GdbmObj::GdbmObj() : has_dbf(0)
{
}

void GdbmObj::Close()
{
	if (has_dbf)
	{
		has_dbf=0;
		gdbm_close(dbf);
	}
}

int GdbmObj::Open(const char *filename, const char *modestr)
{
int	mode=GDBM_READER;

	switch (*modestr)	{
	case 'c':
	case 'C':
		mode=GDBM_WRCREAT;
		break;
	case 'w':
	case 'W':
		mode=GDBM_WRITER;
		break;
	case 'n':
	case 'N':
		mode=GDBM_NEWDB;
		break;
	}

	Close();
	if ((dbf=gdbm_open((char *)filename, 0, mode, 0664, 0)) != 0)
	{
		has_dbf=1;
		return (0);
	}
	return (-1);
}

int GdbmObj::Store(const char *key, size_t keylen,
		const char *data,
		size_t	datalen,
		const	char *mode)
{
datum dkey={(char *)key, keylen};
datum dval={(char *)data, datalen};

	return (has_dbf ? gdbm_store(dbf, dkey, dval, (
			*mode == 'i' || *mode == 'I' ?
				GDBM_INSERT:GDBM_REPLACE)):-1);
}

char	*GdbmObj::Fetch(const char *key, size_t keylen, size_t &datalen,
		const char *options)
{
char	*p;

	for (;;)
	{
		if ((p=do_fetch(key, keylen, datalen)) != 0) return (p);
		if (!options)	break;
		if (*options == 'I')
		{
			while (keylen && key[--keylen] != '.')
				;
			if (!keylen)	break;
			continue;
		}
		if (*options == 'D')
		{
		size_t	i;

			for (i=0; i<keylen; i++)
				if (key[i] == '@') { ++i; break; }
			if (i < keylen)
			{
				if ((p=do_fetch(key, i, datalen)) != 0)
					return (p);
				key += i;
				keylen -= i;
				continue;
			}

			for (i=0; i<keylen; i++)
				if (key[i] == '.') { ++i; break; }
			if (i < keylen)
			{
				key += i;
				keylen -= i;
				continue;
			}
			break;
		}
		break;
	}
	return (0);
}

char	*GdbmObj::do_fetch(const char *key, size_t keylen, size_t &datalen)
{
	if (!has_dbf)	return (0);

datum	dkey={(char *)key, keylen};
datum	val=gdbm_fetch(dbf, dkey);

	if (!val.dptr)	return (0);
	datalen=val.dsize;
	return (val.dptr);
}
