/*
** $Id: rfc2045.h,v 1.3 1999/01/15 00:30:18 mrsam Exp $
*/
#ifndef	rfc2045_h
#define	rfc2045_h

#include	<sys/types.h>
#include	<string.h>
#include	<stdio.h>

#ifdef  __cplusplus
extern "C" {
#endif

struct rfc2045 {
	struct rfc2045 *parent;
	unsigned pindex;
	struct rfc2045 *next;

	off_t	startpos,	/* At which offset in msg this section starts */
		endpos,		/* Where it ends */
		startbody,	/* Where the body of the msg starts */
		endbody;	/* endpos - trailing CRLF terminator */
	char *mime_version;
	char *content_type;
	char *content_transfer_encoding;
	char *content_name;	/* Name of the attachment */
	char *content_boundary;  /* Separate copy, for speed */
	char *charset;
	char *content_disposition;
	char *content_disposition_name;
	char *content_disposition_filename;

	struct rfc2045 *firstpart, *lastpart;

	/* Working area */

	char *workbuf;
	size_t workbufsize;
	size_t workbuflen;
	int	workinheader;
	int	workclosed;
	int	isdummy;
	int	informdata;	/* In a middle of a long form-data part */
	char *header;
	size_t headersize;
	size_t headerlen;

	void	(*decode_func)(struct rfc2045 *, const char *, size_t);
	void	(*udecode_func)(const char *, size_t);
} ;

struct rfc2045 *rfc2045_alloc();
void rfc2045_parse(struct rfc2045 *, const char *, size_t);
void rfc2045_free(struct rfc2045 *);
const char *rfc2045_contentname(const struct rfc2045 *);

void rfc2045_mimeinfo(const struct rfc2045 *,
	const char **,
	const char **,
	const char **);

const char *rfc2045_boundary(const struct rfc2045 *);

void rfc2045_dispositioninfo(const struct rfc2045 *,
	const char **,
	const char **,
	const char **);

void rfc2045_mimepos(const struct rfc2045 *, off_t *, off_t *, off_t *);
unsigned rfc2045_mimepartcount(const struct rfc2045 *);

void rfc2045_xdump(struct rfc2045 *);

struct rfc2045id {
	struct rfc2045id *next;
	int idnum;
} ;

void rfc2045_decode(struct rfc2045 *,
	void (*)(struct rfc2045 *, struct rfc2045id *, void *),
	void *);

struct rfc2045 *rfc2045_find(struct rfc2045 *, const char *);


void rfc2045_cdecode_start(struct rfc2045 *, void (*)(const char *, size_t));
void rfc2045_cdecode(struct rfc2045 *, const char *, size_t);
void rfc2045_cdecode_end(struct rfc2045 *);

void rfc2045_base64encode_start( void (*)(const char *, size_t));
void rfc2045_base64encode(const char *, size_t);
void rfc2045_base64encode_end();

const char *rfc2045_defaultcharset();
struct rfc2045 *rfc2045_fromfp(FILE *);

#ifdef  __cplusplus
}
#endif

#endif
